function t = triangquad(model, cfg)
% Triangular fuzzy approximator for the state; quadratic approximation in u
% Only supports single-action systems
% This approximator type is now obsolete. TRIANGPOLY should be used instead with M=2 (order 2, quadratic)

CFG.type = 'triangquad';            % # of samples to generate
CFG.disc = 0;
CFG.xgrids = [];
CFG.fuzzy_params = {};          % parameters to call problem in fuzzy mode

cfg = parseconfig(cfg, CFG);

if isempty(cfg.xgrids),
    % need to obtain grids from the problem
    fz = feval(model.problem, 'fuzzy', cfg.fuzzy_params{:});
    cfg.xgrids = fz.xgrids;
end;
if isempty(cfg.xgrids),
    error('X grids could not be obtained');
end;

    
t.type = cfg.type;
t.sparse = 1;       % recommend usage of sparse activation vectors
t.uint = [-model.maxu 0 model.maxu];    % this should be changed for non-symmetric U
% core functions
t.phi = @triangquad_phi; t.q = @triangquad_q; t.h = @triangquad_h; 
% plot functions
t.plotv = @triangquad_plotv; t.ploth = @triangquad_ploth;
% centers grids
t.c = cfg.xgrids;
% helper vars for mdegs_c
t.p = model.p;
t.Nc = []; for i = length(t.c):-1:1, t.Nc(i) = length(t.c{i}); end;
t.tab = dec2base(0:2^model.p-1, 2) - 47;
t.roll = 0*t.Nc;
% total number of MFs
t.Ncflat = prod(t.Nc);
% finally, total number of basis functions
t.N = t.Ncflat * 3;

end     % TRIANG constructor


% Compute BF values in x, u
function phi = triangquad_phi(t, x, u)
phi = sparse(t.N, 1);
[ind, mu] = mdegs_p(x, t.c, t.roll, t.Nc, t.p, t.tab);
% phi(x,u) = [phi(x); phi(x) u; phi(x) u^2]
phi(ind) = mu; phi(t.Ncflat + ind) = mu .* u; phi(2*t.Ncflat + ind) = mu .* (u*u); 
end

% Compute Q-value in x, u
function q = triangquad_q(t, theta, x, u)
[ind, mu] = mdegs_p(x, t.c, t.roll, t.Nc, t.p, t.tab); 
mu = mu';
% q(x,u) = phi(x)^T theta_0 + phi(x)^T theta_1 u + phi(x)^T theta_2 u^2
q = mu * theta(ind) ...
    + (mu * theta(t.Ncflat + ind)) * u ...
    + (mu * theta(2*t.Ncflat + ind)) * u*u;
end

% Compute greedy policy in x
function u = triangquad_h(t, theta, x)
[ind, mu] = mdegs_p(x, t.c, t.roll, t.Nc, t.p, t.tab);
mu = mu';
% a = mu * theta(ind);      - the u^0 (bias) coefficient is irrelevant here
b = mu * theta(t.Ncflat + ind);
c = mu * theta(2*t.Ncflat + ind);
if c < 0,           % function is concave, the max is in u0 -- but make sure it's within the interval
    u0 = -b/2/c;
    u = max(min(u0, t.uint(3)), t.uint(1));
elseif c > 0,       % function is convex, max either at start or end of u interval
    u0 = -b/2/c;
    if u0 < t.uint(2),  
        % minimum in the left part of the interval, maximum is therefore at the right end of the
        % interval
        u = t.uint(3);
    else% the other way around
        u = t.uint(1);
    end;
else                % c = 0
    if b > 0,       % linear approximation with positive slope, max at right
        u = t.uint(3);
    else            % the other way around
        u = t.uint(1);
    end;
end;
end

% ----- Plot functions
% Plot V - forward call to generic plot fun
function triangquad_plotv(t, theta, varargin)
for i = length(t.c):-1:1, xb(i, 1:2) = t.c{i}([1 end]); end;
ub = t.uint([1 end]);
approx_plotv(t, theta, xb, ub, varargin{:});
end
% Plot h - forward call to generic plot fun
function triangquad_ploth(t, theta, varargin)
for i = length(t.c):-1:1, xb(i, 1:2) = t.c{i}([1 end]); end;
ub = t.uint([1 end]);
approx_ploth(t, theta, xb, ub, varargin{:});
end

