function varargout = genmpc_exp(mode, cfg)
% Handle experiment in batch run, for this type of algorithm

% 2011-01-31: removed "average" from regret and tree depth, to avoid confusion when a "mean" value
% over random experiments is needed

% TODO implement cleanup

if nargin < 2, cfg = struct; end;

switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'genmpc'};

    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.gamma = [];
        p.X.online.path = 'online'; p.online.shortid = 'onl';
        p.X.path = 'X'; p.X.type = 'vector';
        p.trials = []; p.maxsteps = []; 
        p.reset.path = 'reset'; p.reset.type = 'array'; p.reset.shortid = 'res';
        % planning algorithm and parameters
        p.planalg.path = 'planalg'; p.planalg.shortid = '';
        p.actiontype.path = 'actiontype'; p.actiontype.shortid = 'u';
        p.maxseqlen.path = 'maxseqlen'; p.maxseqlen.shortid = 'maxlen'; p.maxseqlen.label = 'maxseqlen';
        p.maxseqlen.transform = @transformmaxseqlen;    % this is to deal with Inf values
        % generic parameters
        p.planparamsn.path = 'planparams.n'; p.planparamsn.shortid = 'n'; p.planparamsn.label = 'n'; % p.planparamsn.scale = 'log';
        p.planparamsdepth.path = 'planparams.depth'; p.planparamsdepth.shortid = 'd'; p.planparamsdepth.label = 'd'; % p.planparamsdepth.scale = 'log';
        p.planparamsL.path = 'planparams.L'; p.planparamsL.shortid = 'L';
        p.planparamsU.path = 'planparams.U'; p.planparamsU.type = 'vector'; 
        p.planparamsU.formatfun = @formatU; p.planparamsU.shortid = 'M'; % transformed in #actions
        % OPD and OPSS-specific params
        p.planparamsntrans.path = 'planparams.ntrans'; p.planparamsntrans.shortid = 'nt'; p.planparamsntrans.label = 'nt';
        p.planparamscriterion.path = 'planparams.criterion'; p.planparamscriterion.shortid = '';
        p.planparamsactsel.path = 'planparams.actsel'; p.planparamsactsel.shortid = '';
        p.planparamsreusetree.path = 'planparams.reusetree'; p.planparamsreusetree.shortid = 'reusetree';
        p.planparamsreusetree_maxsize.path = 'planparams.reusetree_maxsize'; p.planparamsreusetree_maxsize.shortid = 'rtmaxsize';
        p.planparamsusemem.path = 'planparams.usemem'; p.planparamsusemem.shortid = 'usemem';
        p.planparamsmem_maxsize.path = 'planparams.mem_maxsize'; p.planparamsmem_maxsize.shortid = 'memmaxsize';
        p.planparamslipschitz.path = 'planparams.lipschitz'; p.planparamslipschitz.shortid = 'lip'; % this also applies to LP
        % OLOP-specific params
        p.planparamsexptype.path = 'planparams.exptype'; p.planparamsexptype.type = 'string'; p.planparamsexptype.shortid = 'exp';
        p.planparamsexamineucb.path = 'planparams.examineucb'; p.planparamsexamineucb.type = 'vector';
        % UCT-specific params
        p.planparamsCp.path = 'planparams.Cp'; p.planparamsCp.shortid = 'Cp'; p.planparamsCp.label = 'Cp'; % p.planparamsn.scale = 'log';
        % SOOP- and LP-specific params
        p.planparamsmethod.path = 'planparams.method'; p.planparamsmethod.type = 'string'; p.planparamsmethod.shortid = 'method';
        p.planparamsdimselect.path = 'planparams.dimselect'; p.planparamsdimselect.type = 'string'; p.planparamsdimselect.shortid = 'dimsel';
        p.planparamsalpha.path = 'planparams.alpha'; p.planparamsalpha.shortid = 'alpha'; p.planparamsalpha.label = '\alpha'; 
        % HOLOP-specific params
        p.planparamsK.path = 'planparams.K'; p.planparamsK.shortid = 'K';
        p.planparamsdetact.path = 'planparams.detact'; p.planparamsdetact.shortid = 'detact';
        % we also use planparamslipschitz here
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'model_params', 'reset', 'datadir', 'verb', 'sampledisp', 'trialdisp', 'repeat');
        varargout = {p};
        
%     % provide input variables along which evalbatch will then be able to plot in "inner" mode
%     case 'inputs',
%         in = struct;
%         in.planparamsn.type = 'scalar';
%         in.planparamsn.id = 'planparamsn';
%         in.planparamsn.label = 'n';
%         in.planparamsn.val = [];
% 
%         in.planparamsmem_maxsize.type = 'scalar';
%         in.planparamsmem_maxsize.id = 'planparamsmem_maxsize';
%         in.planparamsmem_maxsize.label = 'memsize_{max}';
%         in.planparamsmem_maxsize.val = [];
% 
%         varargout = {in};
        
    % provide output variables
    case 'outputs',
        o = struct;
        
        % for offline: average regret over all states considered
        o.regret.id = 'regret';
        o.regret.label = 'regret';
        o.regret.params = {'X', 'Qstar', 'hstar'};
        o.regret.scale = 'log';

        % for online: average return for all initial states
        o.return.id = 'return';
        o.return.label = 'return';
        % o.return.params = {'X', 'Qstar', 'hstar'};
        % o.return.scale = 'log';

        % total execution time
        o.exectime.id = 'exectime';
        o.exectime.label = 'exec. time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';

        % average planning time per state; for online: the same as the planning time/step
        o.plantime.id = 'plantime';
        o.plantime.label = 'planning time';
        o.plantime.units = 's';
        o.plantime.scale = 'log';

        % average tree depth
        o.depth.id = 'depth';
        o.depth.label = 'tree depth';

        % average planning horizon (for some aglos like OPD, OPSS, this is the same as depth)
        o.horizon.id = 'horizon';
        o.horizon.label = 'horizon';
        % o.horizon.scale = 'log';

        varargout = {o};
        
    % process options
    case {'run'},
        cfg.(mode) = 1;
        % disp('Dummy call genmpc with config:'); dispx(cfg, 0, 0, 'cfg', 'config'); cfg.planparams(1)
        genmpc(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'timestat');
        varargout = {timestat.run};
        
    % compute average planning time per state
    case 'plantime',
        load(cfg.datafile, 'timestat');
        % make column vector for easy handling
        plantime = timestat.planning(:);
        % note that, for online case, there may be noninitialized (nan) planning times; ignore them
        varargout = {mean(plantime(~isnan(plantime)))};

    % find average tree depth
    case 'depth',
        load(cfg.datafile, 'planinfoh');
        switch cfg.planalg,
            case {'opd', 'opss'},
                varargout = {mean([planinfoh(:).depth])};
            otherwise
                error('Can''t compute depth for algorithm [%s]', cfg.planalg);
        end;

    % find average planning horizon: note for some algos this is identical to 'depth' above
    case 'horizon',
        load(cfg.datafile, 'planinfoh');
        switch cfg.planalg,
            case {'opd', 'opss'},
                varargout = {mean([planinfoh(:).depth])};
            case 'sooplp'
                varargout = {mean([planinfoh(:).longest])};
            otherwise
                error('Can''t compute planning horizon for algorithm [%s]', cfg.planalg);
        end;

    % evaluate regret
    case 'regret',
        % load experiment config
        loadwithprefix(cfg.datafile, 'e_', 'cfg');
        % error checks:
        if e_cfg.online, error('Regret computation: offline solution required');
        % the test below will not work as the states may be expanded to numerical values
        % we must be called with the same X with which genmpc was run offline
        % elseif ~isequal(e_cfg.X, cfg.X), error('Regret computation: X must be the same as in GENMPC'); 
        end; 
        % parameters on calling config
        srcfg = struct;
        srcfg.hstar = cfg.hstar;
        srcfg.Qstar = cfg.Qstar;
        srcfg.X = cfg.X;
        % take problem from experiment config
        srcfg.problem = e_cfg.problem;
        % build policy: simply looks up the value of x in X, returns corresponding U
        load(cfg.datafile, 'X', 'U');
        srcfg.h = @(x) lookuph(x, X, U);
        % find and return regret
        [avgreg, reg, info] = simpleregret(srcfg);
        varargout = {avgreg, reg, info};
        
    % evaluate return
    case 'return',
        % load experiment config
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'R'});
        % error checks:
        if ~e_cfg.online, error('Return computation: online solution required'); end; 
        ntrials = size(e_R, 3); ret = zeros(ntrials, 1);
        for trial = 1:ntrials,
            K = find(~isnan(e_R(1, :, trial)), 1, 'last') - 1;
            ret(trial) = sum((e_cfg.gamma .^ (0:K-1)) .* e_R(1, 2:K+1, trial));
        end;
        varargout = {mean(ret)};
        
    otherwise,  % return empty
        varargout = {[]};
end;

end     % main function genmpc_exp

% %%%%%%%% local funs
% formatting, transforms
% maxseqlen: if fractional and infinite values coexist, then Inf = 1
function val = transformmaxseqlen(val), if any(val < 1), val(~isfinite(val)) = 1; end; end
% U: format as count of actions
function val = formatU(val), val = num2str(size(val, 2)); end

% Local function for lookup-policy (used in the regret branch above)
function u = lookuph(x, X, U)
    i = findflat(x, X, 1, 'first');
    if isempty(i), error('State not part of the set considered by GENMPC'); end;
    u = U(:, i);
end
