function varargout = lspi_exp(mode, cfg)
% Handle experiment in batch run, for this type of algorithm

% TODO implement cleanup

if nargin < 2, cfg = struct; end;

switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'lspi'};
    
    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.gamma = [];
        % approximator parameters
        p.approxenhanced.path = 'approx.enhanced';
        p.approxtype.path = 'approx.type'; p.approxtype.shortid = '';
        p.approxN.path = 'approx.N'; p.approxN.type = 'vector'; p.approxN.shortid = 'N';
        p.approxxspace.path = 'approx.xspace'; p.approxxspace.shortid = '';
        p.approxM.path = 'approx.M'; p.approxM.type = 'vector'; p.approxM.shortid = 'M';
        p.approxuspace.path = 'approx.uspace'; p.approxuspace.shortid = '';
        % sample parameters
        p.samplesmethod.path = 'samples.method'; p.samplesmethod.shortid = '';
        p.samplesN.path = 'samples.N'; p.samplesN.shortid = 'N';
        p.samplesgridfun.path = 'samples.gridfun'; p.samplesgridfun.shortid = 'fun';
        p.samplesxgrids.path = 'samples.xgrids'; p.samplesxgrids.type = 'vector'; p.samplesxgrids.shortid = 'xgr';   
            % note we assume we won't explicitly use cell arrays of points, 
            % thus allowing for multiple settings using cell array of vectors
        p.samplesugrids.path = 'samples.ugrids'; p.samplesugrids.type = 'array'; p.samplesugrids.shortid = 'ugr';
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'method', 'invdelta', 'maxiter', 'eps', 'epsper', 'maxper', 'nper', ...
            'datadir', 'verb', 'iterdisp', 'itersave');
        varargout = {p};
        
    % provide output variables
    case 'outputs',
        o = struct;
        o.score.id = 'score';
        o.score.label = 'score';
        o.score.params = {'gamma', 'X0', 'mc_eps', 'mc_N'};
        
        o.exectime.id = 'exectime';
        o.exectime.label = 'execution time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';

        o.iter.id = 'iter';
        o.iter.label = 'number of iterations';
        
        varargout = {o};
        
    % process options
    case {'run', 'sol'},
        cfg.(mode) = 1;
        lspi(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'timestat');
        varargout = {timestat.trun};

    % find # iterations to completion
    case 'iter',
        load(cfg.datafile, 'k');
        varargout = {k-1};

    % evaluate score
    case 'score',
        load(cfg.datafile, 'model', 'approx', 'theta');
        mccfg.gamma = cfg.gamma;
        mccfg.mc_eps = cfg.mc_eps;
        mccfg.mc_N = cfg.mc_N;
        % if X0 is string, call problem function to get actual states
        if ischar(cfg.X0), cfg.X0 = feval(model.problem, 'X0', cfg.X0); end;
        score = mc_approx(model, approx, theta, cfg.X0, mccfg);
        varargout = {score};
        
    otherwise,  % return empty
        varargout = {[]};
end;

