function app = nonparamdisc(model, cfg)
% OBSOLETE: USE REGDISC INSTEAD
% Approximator composed of distinct, nonparametric approximators
% for each discrete action

CFG.type = 'nonparamdisc';
CFG.disc = 1;
CFG.U = [];         % cell array of grids
CFG.regmethod = [];
CFG.regressors = [];

app = parseconfig(cfg, CFG);

if isempty(app.U),
    error('NONPARAMDISC requires predefined discrete actions in field Uflat');
end;

switch app.regmethod,
    case 'extratrees',
    otherwise,
        error('NONPARAMDISC does not support regression method [%s]', app.regmethod);
end;
app.Uflat = flat(app.U);
app.maxx = model.maxx; app.maxu = model.maxu;
% number of action variables and discrete actions
app.dimx = model.p; app.dimu = model.q;
app.M = size(app.Uflat, 2);

% there is no "activation degree" function because the approximator is
% nonparametric
app.phi = []; 
app.q = @nonparamdisc_q; app.h = @nonparamdisc_h; 
% plot functions
app.plotv = @nonparamdisc_plotv; app.ploth = @nonparamdisc_ploth;
app.vectorized = 1;    % signal approximator supports vectorized q and h calls

end     % TRIANG constructor

% ===== Core functions

% Q-value in x, u (theta is ignored, approx is nonparametric)
% supports vectorized x and u (the # of points should obviously match)
function q = nonparamdisc_q(app, theta, x, u)
Np = size(x, 2);
if Np == 1,
    ui = findflat(u, app.Uflat, 1, 'first');    
    q = rtenspred(app.regressors{ui},single(x'));
else
    % vectorized case, make one call for every discrete action present in
    % the samples
    q = zeros(1, Np);
    for j = 1:app.M, 
        is = findflat(app.Uflat(:, j), u);
        if isempty(is), continue; end;
        q(is) = rtenspred(app.regressors{j},single(x(:, is)'));
    end;
end;
end

% Greedy policy in x (theta is ignored, approx is nonparametric)
% Supports vectorized x. Also returns the Q-values of the optimal actions
function [u, qmax] = nonparamdisc_h(app, theta, x)
Np = size(x, 2);
q = zeros(Np, app.M);
% compute approximate Q values for all the states, for each action in turn
for j = 1:app.M, q(:, j) = rtenspred(app.regressors{j}, single(x')); end;
% find the Q-value maxima and set the actions accordingly 
u = zeros(app.dimu, Np);
[qmax, jmax] = max(q, [], 2);
% this for could be eliminated, but it's not the most expensive...
for i = 1:Np, u(:, i) = app.Uflat(:, jmax(i)); end;
end

% ----- Plot functions
% Plot V - forward call to generic plot fun
function nonparamdisc_plotv(app, theta, varargin)
approx_plotv(app, [], [-app.maxx app.maxx], [-app.maxu app.maxu], varargin{:});
end
% Plot h - forward call to generic plot fun
function nonparamdisc_ploth(app, theta, varargin)
approx_ploth(app, [], [-app.maxx app.maxx], [-app.maxu app.maxu], varargin{:});
end
