%% Demonstration of DP/RL algorithms on the cleaning robot problem
% Initialization

% common config
ccfg.target = 'screen';
ccfg.problem = 'machrepl_problem';
pparam.n = 5;
pparam.rew_c = 1;
ccfg.model_params = {pparam};
ccfg.gamma = .9;
ccfg.eps = .001;
ccfg.viscfg = struct;
ccfg.viscfg.figsize = [600 600];
ccfg.viscfg.pause = -1;

% to get only console output for LaTeX, use this:
% ccfg.verb = -Inf; ccfg.viscfg.consoleout = 1;

% retrieve (or, if not available, compute) optimal solution with policy iteration
cfg = ccfg;
cfg.datafile = 'machrepl_optsol_pi';
if ~exist([cfg.datafile '.mat'], 'file'),
    cfg.run = 1;
    cfg.verb = -Inf;    % keep things silent
    cfg.peval_eps = 1e-5;   
    cfg.eps = 0;
    piter(cfg);
end;
load(cfg.datafile, 'Q', 'h', 'Qh');
ccfg.viscfg.Qstar = Q;
ccfg.viscfg.hstar = h;
ccfg.viscfg.Qhstar = Qh;

% common parameters changes for online learning
onlcfg = ccfg;
onlcfg.eps = .01;
onlcfg.reset = 1;
onlcfg.explor = .3;
onlcfg.explordecay = 1;
onlcfg.alpha = .1;
onlcfg.alphadecay = 1;
onlcfg.maxsteps = 20;
onlcfg.maxtrials = 30;
onlcfg.viscfg.pause = {'cfg.trial == 1 && cfg.k <= 5', -1; 'true', 0};



%% Q-iteration
cfg = ccfg;
cfg.visualize = 1;
cfg.run = 1;
cfg.viscfg.pause ={'cfg.ell <= 5', -1; 'true', 0};
qiter(cfg);


%% Policy iteration with Q-functions
cfg = ccfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.peval_eps = 0.01;
cfg.viscfg.pause ={'(cfg.ell == 1 && cfg.tau <= 5) || (cfg.piter)', -1; 'true', 0};
piter(cfg);

% 
%% SARSA
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.randseed = 10;        % uncomment for repeatable results
sarsa(cfg);


%% Q-learning
cfg = onlcfg;
cfg.visualize = 2;
cfg.run = 1;
cfg.randseed = 10;        % uncomment for repeatable results
qlearn(cfg);

% %% SARSA(lambda)
% cfg = onlcfg;
% cfg.visualize = 2;
% cfg.run = 1;
% cfg.lambda = .5;
% cfg.randseed = 1000;      % uncomment for repeatable results
% sarsa(cfg);
% 
% 
% 
% %% Q(lambda)-learning
% cfg = onlcfg;
% cfg.visualize = 2;
% cfg.run = 1;
% cfg.lambda = .5;
% cfg.randseed = 10;      % uncomment for repeatable results
% qlearn(cfg);
% 
% 
% %% ER-Q-learning
% cfg = onlcfg;
% cfg.visualize = 3;
% cfg.run = 1;
% cfg.maxtrials = 40;
% cfg.randseed = 10;        % uncomment for repeatable results
% cfg.ertype = 'fwd';
% cfg.erN = 5;
% cfg.viscfg.pause = {'any(cfg.trial == 3)', -1; 'true', 0};
% qlearn(cfg);
% 
% 
% 
