function figh = dc_optimallqr(cfg)
% Evaluates optimal LQR controller for the DC model 
% 2009-02-25: Updated to support new-style policy with t argument (and replaced
% linearstatefeedback with lsf)

% Default model is linear -- the only one for which the
% LQR controller is really optimal, and even then modulo the constraints

% default arguments
CFG.eval = 0;               % eval score
CFG.X0 = 'coarse';
CFG.eps = 0.1;
% plot config
CFG.slice = [NaN NaN 0];
CFG.gridres = 100;
CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG
CFG.plottarget = 'screen';  % 'latex', 'beamer', 'screen', ''
CFG.savedir = ''; 
CFG.model_params = {'lqr', 'lin'};
CFG.hcolorbar = 1;
CFG.plotq = 0;
% process config
cfg = parseconfig(cfg, CFG);

model = dc_problem('model', cfg.model_params{:});

% compute discounted LQR state feedback
dcfg.gamma = model.gamma;
dcfg.model = model;

[K, X, dcfg] = gammadare(dcfg);

% K
% figh = 0; return

if cfg.eval,
    scfg.X0 = dc_problem('X0', cfg.X0);
    scfg.maxr = model.maxr;
    scfg.gamma = model.gamma;
    scfg.eps = cfg.eps;
    scfg.policy = @lsf;         % linear state feedback with saturation on the control
    scfg.policyargs = {K, model.maxu};
    [J, R] = eval_policy(scfg, model);
    optimalscore = J
end;    % last result:optimalscore =~ -203.2

% Plot optimal policy
pcfg = cfg;         % copy relevant options
pcfg.datasource = 'caller';
pcfg.genh = 1;
pcfg.savefig = 'dc_lqr_hstar';
pcfg.policy = @lsf;
pcfg.policyargs = {K, model.maxu};
if cfg.hcolorbar, pcfg.colorbar = 1; end;
figh = dc_plot(pcfg);

if cfg.plotq,
    % Plot optimal Q-function (trouble with constraints!)
    pcfg = cfg;         % copy relevant options
    pcfg.datasource = 'caller';
    pcfg.genq = 1;
    pcfg.savefig = 'dc_lqr_qstar';
    pcfg.qfunction = @lqrq;
    pcfg.qargs = {X, model.gamma, model};
    figh = [figh dc_plot(pcfg)];
end;

end

