function dd_cerbfps(mode, c, exp)
% Cross-entropy policy search for the discrete double integrator problem.

% What experiments to process and in what mode (defaults)
if nargin < 1, mode = 'run'; end;
if nargin < 2, c = struct; end;
if nargin < 3, exp = 'oldsat'; end;

% defaults
CFG = struct;
CFG.(mode) = 1;
CFG.startat = 1; CFG.repeat = 20;
CFG.problem = 'ddi_problem';
CFG.gamma = 0.95;
CFG.alg = 'cerbfps';
CFG.N = 4:18;
CFG.X0 = 'p01s02';
CFG.datadir = pwd;
CFG.verb = 2;
CFG.int = 'conf';
CFG.conftype = 'gauss';
% Note below changes from thesis&older experiments: 
% there c=5, rho=.05, alpha=1 (no smooth), eps=.005
CFG.ce_c = 10;     
CFG.ce_rho = 0.01;
CFG.ce_alpha = 0.7;
CFG.ce_eps = 0.001;
CFG.mc_maxsteps = 0.001;    
% Note mc_maxsteps was erroneously called "mc_eps" when running the experiments...
% which means mc_maxsteps stayed at the default value of .005.
% This does not make much difference since near-optimal trajectories for this problem finish
% much earlier than the upper bound given by either .005 or .001

% below are simply transmitted
CFG.datadir = 'd:\work\m\approxrl\systems\discretedi\dd_cerbfps\';
CFG = setfigprop(CFG, 'addfields');
CFG.Jstar = [];

% process config
ccfg = parseconfig(c, CFG);


%% Experiment 1: old saturation style
%   - run in Feb 2009, 
%   - with the updated version of CERBFPS 
%   - with new settings for the parameters (finer thresholds and larger #samples mainly)
%       - but see bug below
%   - on the new DCSC desktop machines (Dell Optiplex 755)
cfg = ccfg;
if checkrunexp('oldsat', exp), optimexp(cfg); end;


%% Experiment 1: no saturation
% May 2010
cfg = ccfg;
cfg.model_params = {'QUAD2', 1, 'nosat'};
cfg.infix = 'nosat';
if checkrunexp('nosat', exp), optimexp(cfg); end;
