function dd_optrbfps(exper, mode, c)
% Generic-optimization policy search for the discrete double integrator problem.

% What experiments to process and in what mode (defaults)
if nargin < 1, exper = 'norest'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% ==== init defaults
CFG = struct;
CFG.(mode) = 1;
CFG.repeat = 1;
CFG.problem = 'ddi_problem';
CFG.gamma = 0.95;
CFG.alg = 'optrbfps';
CFG.solver = 'glcFast';
CFG.N = 4:18;
CFG.X0 = 'p01s02';
CFG.mc_maxsteps = 0.001;

% output config
CFG.datadir = pwd;
CFG.verb = 3;
CFG.int = 'conf';
CFG.conftype = 'gauss';

CFG = setfigprop(CFG, 'addfields');
CFG.Jstar = [];

% ==== process config
ccfg = parseconfig(c, CFG);


% No-restarts experiment
nor = ccfg;
nor.miscinfo = 'norest';
nor.infix = 'norest';
nor.maxrestarts = 1;
if checkrunexp(nor.miscinfo, exper), optimexp(nor); end;


% No-restarts, "nosat"=terminal state handling fixed
nor = ccfg;
nor.miscinfo = 'nosat';
nor.infix = 'nosat';
nor.maxrestarts = 1;
nor.model_params = {'QUAD2', 1, 'nosat'};
if checkrunexp(nor.miscinfo, exper), optimexp(nor); end;

% Restarts experiment
rest = ccfg;
rest.miscinfo = 'rest';
rest.infix = 'rest';
rest.maxstarts = 5;
rest.eps = .001;
if checkrunexp(rest.miscinfo, exper), optimexp(rest); end;


