function rtrobot_p(Port,ref,Kp)
% Control the Ed-Ro robot to a desired pose (0 by default)
%    RTROBOT_P(Port,ref,Kp)
%
%       Port ... port, e.g., 'COM1' (default Port = 'COM1')
%       ref  ... reference to initialize to (default ref = 0)
%       Kp   ... proportional gain (default Kp = 3)

% (c) Robert Babuska, 2005

joint = [5];                                % index of joint(s) to be controlled
                                            % (1 = base, 2 = shoulder, etc.)
                                            % can be a vector too
n = length(joint);                          % number of joints to be controlled
dbcontjoint = joint(:)+(0:6:6*(n-1))';            % convert into sequential index
joint_select = zeros(6,n);                  % prepare zeros
joint_select(joint) = 1;                    % assing 1 at the right places

if nargin < 1, Port = 'COM8'; end;              % default port
if nargin < 2, ref = 0; end;                    % default reference
if nargin < 3, Kp = 3; end;                     % default control gain

h = 0.05;                                       % sampling period
eps = 0.05;                                     % tolerance around ref
umax = 0.5;                                     % maximum allowed control 

xx = zeros(2,7);                                % aux var for BE conversion

% initialize, open the port, define timer
hwinit;                                         % read HW parameters
daoutgain = daoutgain(1:6);                     % ignore laser
daoutoffs = daoutoffs(1:6);                     % ignore laser
com = rs232('GetParams','default');             % get default RS232 parameters
com.Port = Port;                                % define port number
com.BaudRate = 115200;                          % define Baud rate
com.ReadTimeout = 1;                            % define short timeout
com.WriteTimeout = 1;                           % define short timeout
[params,result]=rs232('open',com);              % open the port

% control loop
%tic;
for i = 1 : 50,                                 % run for at most 40*h seconds
    t = clock;
    rs232('write',com,uint8(128));              % control byte for read D/A
    % could also read only one byte if only one joint is controlled (but need control code from Robert)
    xx(:) = rs232('read',com,14);               % read 14 bytes from 7 sensors
    x = double(xx'*[256 1]');                   % convert to big endian
    x = adingain(:).*(x + adinoffs(:));         % scale to physical range
    x = joint_select'*x(1:6);                   % select the right variable   
    % should be removed, expanding vector => computationally inefficient
    xlog(i,:) = x';
    u = min(umax,max(-umax,Kp.*(ref-x)));       % saturate control action
    ulog(i,:) = u';
%     u = 0;
    u = joint_select*u;                         % complete control vector
    u = daoutgain(:).*u + daoutoffs(:);         % scale to digital range
    rs232('write',com,uint8(0));                % control byte for send all actuators
    rs232('write',com,uint8(u)');               % write 6 bytes to 6 actuators
    while all(clock-t < h-0.1*h), end;          % wait for the next sample
end;
%toc

% reset actuators, close the port
rs232('write',com,uint8(56));                   % laser off
rs232('write',com,uint8(0));                    % control byte for send all actuators
rs232('write',com,uint8(127*ones(6,1)));        % reset all 6 actuators to 0 (127 bin)
rs232('close',com);                             % close serial port

assignin('base','x',xlog);
assignin('base','u',ulog);