function rtrobot_p1(Port,ref,Kp)
% Control the Ed-Ro robot to a desired pose (0 by default)
%    RTROBOT_P(Port,ref,Kp)
%
%       Port ... port, e.g., 'COM1' (default Port = 'COM1')
%       ref  ... reference to initialize to (default ref = 0)
%       Kp   ... proportional gain (default Kp = 3)

% (c) Robert Babuska, 2005

joint = [5];                                % index of joint(s) to be controlled
                                            % (1 = base, 2 = shoulder, etc.)
                                            % can be a vector too
n = length(joint);                          % number of joints to be controlled
joint = joint(:)+(0:6:6*(n-1))';            % convert into sequential index
joint_select = zeros(6,n);                  % prepare zeros
joint_select(joint) = 1;                    % assing 1 at the right places

if nargin < 1, Port = 'COM1'; end;              % default port
if nargin < 2, ref = 0; end;                    % default reference
if nargin < 3, Kp = 3; end;                     % default control gain

h = 0.001;                                      % sampling period
eps = 0.05;                                     % tolerance around ref
umax = 0.5;                                     % maximum allowed control 

xx = zeros(2,1);                                % aux var for BE conversion

% initialize, open the port, define timer
hwinit;                                         % read HW parameters
daoutgain = daoutgain(joint);                   % select joint
daoutoffs = daoutoffs(joint);                   % select joint
adingain = adingain(joint);                     % select joint
adinoffs = adinoffs(joint);                     % select joint
com = rs232('GetParams','default');             % get default RS232 parameters
com.Port = Port;                                % define port number
com.BaudRate = 115200;                          % define Baud rate
com.ReadTimeout = 1;                            % define short timeout
com.WriteTimeout = 1;                           % define short timeout
[params,result]=rs232('open',com);              % open the port

rtclear all;
rtdef(1,'tick',h);
rtstart(1);

% control loop
tic;
for i = 1 : 1000,                               % run for at most 40*h seconds
%    t = clock;
    rs232('write',com,uint8(196));              % control byte for read one D/A
    xx(:) = rs232('read',com,2);                % read 2 bytes from 1 sensors
    x = double(xx'*[256 1]');                   % convert to big endian
    x = adingain.*(x + adinoffs);               % scale to physical range
    u = min(umax,max(-umax,Kp.*(ref-x)));       % saturate control action
    u = daoutgain.*u + daoutoffs;               % scale to digital range
    rs232('write',com,uint8(68));               % control byte for send one actuator
    rs232('write',com,uint8(u)');               % write 1 byte to 1 actuator
    while rtrd(1,'t') < i, end;                 % wait for the next sample
%    while all(clock-t < h-0.1*h), end;          % wait for the next sample
end;
toc

% reset actuators, close the port
rs232('write',com,uint8(56));                   % laser off
rs232('write',com,uint8(0));                    % control byte for send all actuators
rs232('write',com,uint8(127*ones(6,1)));        % reset all 6 actuators to 0 (127 bin)
rs232('close',com);                             % close serial port
