% File for running HIV control experiments with optimistic planning
function hiv_op(mode, expids, customcfg, customplanparams)

% main configs
% model config
MCFG = struct;
MCFG.filter = 'none';
MCFG.rewtype = 'normadams';  % normalized!
ODECFG = struct;
% algo config
CFG = struct;
CFG.planparams.U = flat({[0, 0.7], [0, 0.3]});
CFG.trials = 1;
CFG.reset = 'unhealthy';
CFG.maxsteps = 800/5;
CFG.verb = 3;
CFG.datadir = pwd;

% default custom configs (empty)
if nargin < 3, customcfg = struct; end;
if nargin < 4, customplanparams = struct; end;

    % nested utility function for executing experiment
    function execexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            cfg = parseconfig(customcfg, cfg);
            cfg.planparams = parseconfig(customplanparams, cfg.planparams);
            % check if we need to call 'process' or 'eval'
            cfg.(mode) = 1;
            genmpc(cfg);
        end;
    end


%% Deterministic, optplan
mcfg = MCFG; 
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'optplan';
% cfg.planparams.criterion = 'contrib';
% cfg.planparams.actsel = 'util';
cfg.planparams.n = 1000;
cfg.datafile = 'hiv_op_det_optplan_n1000';

execexp('optplan');


%% Stochastic, OPSS
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opss';
cfg.planparams.criterion = 'contrib';
cfg.planparams.actsel = 'util';
cfg.planparams.L = 4;
cfg.planparams.n = 3000;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_opss_n3000_p05g09p05g11_T1100';

execexp('opss');

%% Stochastic, uniform
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opss';
cfg.planparams.criterion = 'unif';
cfg.planparams.actsel = 'util';
cfg.planparams.L = 4;
cfg.planparams.n = 3000;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_unif_n3000_p05g09p05g11_T1100';

execexp('unif');

%% Stochastic, OLOP
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'olop';
cfg.planparams.n = 3000 * 4 * size(CFG.planparams.U, 2);
cfg.planparams.verb = 1;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_olop_n48000_p05g09p05g11_T1100';

execexp('olop');

%% Stochastic, OPSS with memory reuse
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opss';
cfg.planparams.criterion = 'contrib';
cfg.planparams.actsel = 'util';
cfg.planparams.L = 4;
cfg.planparams.n = 1000;
cfg.planparams.usemem = 1;
cfg.planparams.mem_maxsize = 1;     % 1 * tree size
cfg.planparams.lipschitz = 0.1;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_opss_n1000_usemem_lip01_p05g09p05g11_T1100';

execexp('usemem');

end     % main function hiv_op