function lspeos_dupd(exp, mode, cfg)
% Study of dupd influence for LSPE online/optimistic

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'choosealpha'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, cfg = struct; end;

% === DEFAULT CONFIG
CFG = struct;
CFG.(mode) = 1;
CFG.startat = 1; CFG.repeat = 20;

% LSPE algorithm config, via EXP meta-config fields 
% (including approx, problem, etc.)
CFG.alg = 'lspeonline';
CFG.problem = 'ipsetup_problem';
CFG.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
CFG.gamma = 0.98;
CFG.dupd = [1 10 100 1000 5000] * 0.005;    
CFG.type = 'rbfdisc'; CFG.N = 11; CFG.M = 3; CFG.xspace = 'eq'; CFG.uspace = 'log';
CFG.trialtime = 1.5;                        % sufficient for stabilization 
CFG.lspi_params.maxtime  = 60*10;           % 10 simulated minutes
CFG.lspi_params.explor = 1;                 % start with this exploration rate
    % and decay so that we have 0.1 at the end of the experiment:
CFG.explordecay = 0.1^(1/CFG.lspi_params.maxtime);  
% stats config
CFG.lspi_params.dstoretheta = 1;
CFG.lspi_params.verb = 5;
CFG.lspi_params.ddisp = 10;
CFG.lspi_params.dsave = 60;

% EXP meta-config
% data files
CFG.datadir = pwd; CFG.show = {'dupd', 'alpha', 'rep'}; CFG.hide = {'approx'};
% perf eval
CFG.X0 = 'small'; CFG.perf = 1; CFG.mc_eps = 0.1; 
CFG.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
CFG.forceeval = 1;                         % in STATS mode -- since no exp actually converges
% replay
CFG.x0 = [-pi;0]; CFG.tend = 3; 
% plot
CFG.savedir = '';   % dummy so that this value can be set by user
CFG.plottarget = 'screen'; CFG.savefig = 'dupd';
CFG.plotconf = 1; CFG.plotminmax = 0; CFG.minmaxtype = 'area';
CFG.fixedfigname = 0;
CFG.addtocrt = [];
CFG.stats = 0;
CFG.plot = [];
CFG.perf = 0;
%     CFG.figsize = [450 400];
if strcmp(mode, 'onlineperf'),      CFG.ylim = [-2700 -1400];
elseif strcmp(mode, 'onlineconv'),  CFG.ylim = [10^0 10^10];    
end;
if any(strcmp(mode, {'onlineperf', 'onlineconv'})), CFG.verb = 0; else CFG.verb = 5; end;

% === INSTALL CUSTOM CONFIG SETTINGS
cfg = parseconfig(cfg, CFG);

expout = [];   % dummy


% === EXPERIMENT TO CHOOSE ALPHA
% These experiments were run on the new TU desktop computer
% NOTE that the experiments with file names "aXXX_dupd05" are no longer needed
% (their data should be removed from the cache)
caexp = cfg;
caexp.miscinfo = 'choosealpha';
caexp.infix = 'choosealpha';
caexp.repeat = 1;       % do a single pass (this is a preliminary experiment)
% test these values of alpha (stepsize)
caexp.alpha = [.001 .005 .01 .05 .1 .5 1];
% for every dupd in the set configured above, default:
% caexp.dupd = [1 10 100 1000 5000] * 0.005;    


% From these results it appears that alpha=0.1 might be a reasonable compromise between
% learning speed and stability (but see below)
% (1 EXPER AVAILABLE FOR EACH)
if checkrunexp(caexp.miscinfo, exp), 
    switch mode,
        case {'onlineperf', 'onlineconv'},
            % make separate plots for each value of dupd 
            dupd = caexp.dupd;
            for i = 1:length(dupd),
                caexp.dupd = dupd(i); expout = lspiexp(caexp);
                title(['T_{\theta}=' sprintf('%.3f', dupd(i))]);
                set(gcf, 'Name', sprintf('dupd=%.3f', dupd(i)));
            end;
        otherwise,
            expout = lspiexp(caexp); 
    end;
end;


% === "REAL" STUDY OF INFLUENCE OF DUPD, FOR A FIXED ALPHA
% These experiments were run on the new TU desktop computer
duexp = cfg;
duexp.miscinfo = 'dupd';
duexp.infix = 'dupd';
% for every dupd in the set configured above, default:
% duexp.dupd = [1 10 100 1000 5000] * 0.005;    
% corresponding alpha values for each value of dupd (20 EXPERS AVAILABLE)
duexp.alpha = [.005 .01 .01 0.1 0.1];
% the values of alpha above should be used for the respective dupd experiments;
% do not run experiments for crossproduct of alpha and dupd:
duexp.donotexpand = {'alpha'};
% % older experiment used the same alpha, .01, for every dupd (10 EXPERS AVAILABLE)
% duexp.alpha = .01;
if checkrunexp(duexp.miscinfo, exp), expout = lspiexp(duexp); end;

% two selected values with confidence levels; should only be used in plot mode
duexp = cfg;
duexp.miscinfo = 'dupdconf';
duexp.infix = 'dupd';
duexp.dupd = [1 5000] * 0.005;    
duexp.alpha = [.005 0.1];
duexp.donotexpand = {'alpha'};
if checkrunexp(duexp.miscinfo, exp), expout = lspiexp(duexp); end;


% === CHANGED EXPLORATION RATE: PRELIMINARY EXPERIMENT TO CHOOSE ALPHA
caexp = cfg;
caexp.miscinfo = 'choosealphaB';
caexp.infix = 'choosealphaB';
caexp.explordecay = 0.1^(1/200);
caexp.repeat = 3;       % do few passes only (this is a preliminary experiment)
% test these values of alpha (stepsize)
caexp.alpha = [.001 .01 .1 1];
% for every dupd in the set configured above, default:
% caexp.dupd = [1 10 100 1000 5000] * 0.005;    
% use periodic resets to a fixed state, to also be able to 
% evaluate the "onlinereturn"
caexp.lspi_params.reset = 'randfixed';
caexp.lspi_params.x0 = [-pi; 0];
caexp.lspi_params.resetper = 10;

if checkrunexp(caexp.miscinfo, exp), 
    switch mode,
        case {'onlineperf', 'onlineconv', 'onlinereturn'},
            % make separate plots for each value of dupd 
            dupd = caexp.dupd;
            for i = 1:length(dupd),
                caexp.dupd = dupd(i); expout = lspiexp(caexp);
                title(['T_{\theta}=' sprintf('%.3f', dupd(i))]);
                set(gcf, 'Name', sprintf('dupd=%.3f', dupd(i)));
            end;
        otherwise,
            expout = lspiexp(caexp); 
    end;
end;

% === CHANGED EXPLORATION RATE, STUDY OF INFLUENCE OF DUPD, FOR A FIXED ALPHA
% These experiments were run on the new TU desktop computer
duexp = cfg;
duexp.miscinfo = 'dupdB';
duexp.infix = 'dupdB';
duexp.explordecay = 0.1^(1/200);
% for every dupd in the set configured above, default:
% duexp.dupd = [1 10 100 1000 5000] * 0.005;    
% corresponding alpha values for each value of dupd (20 EXPERS AVAILABLE)
duexp.alpha = [.001 .01 .001 .01 .1];
if isfield(cfg, 'alpha'), duexp.alpha = cfg.alpha; end;
% the values of alpha above should be used for the respective dupd experiments;
% do not run experiments for crossproduct of alpha and dupd:
duexp.donotexpand = {'alpha'};
if checkrunexp(duexp.miscinfo, exp), expout = lspiexp(duexp); end;


duexp = cfg;
duexp.miscinfo = 'dupdBconf';
duexp.infix = 'dupdB';
duexp.explordecay = 0.1^(1/200);
% for every dupd in the set configured above, default:
duexp.dupd = [1 5000] * 0.005;    
% corresponding alpha values for each value of dupd (20 EXPERS AVAILABLE)
duexp.alpha = [.001 .1];
% the values of alpha above should be used for the respective dupd experiments;
% do not run experiments for crossproduct of alpha and dupd:
duexp.donotexpand = {'alpha'};
if checkrunexp(duexp.miscinfo, exp), expout = lspiexp(duexp); end;