function lspios_dupdstudy(exp, mode, c)

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'ronl'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
% --------------
% main LSPIEXP config
cfg.(mode) = 2;
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.98;
% misc config (output etc)
cfg.useprefixes = 0;
cfg.datadir = pwd;
cfg.eps = 0;                            % zero convergence threshold (force until real convergence)
% approximator main config
cfg.N = 11;
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
cfg.datadir = pwd;
cfg.dupd = [];
cfg.Ns = 20000;
% perf eval
cfg.forcerecompute = 0;
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.savedir = '';   % dummy so that this value can be set
cfg.plottarget = 'screen'; cfg.savefig = 'dupd';
cfg.fixedfigname = 0;
cfg.plotfun = @plot;
cfg.plotconf = 0; cfg.plotminmax = 0; cfg.minmaxtype = 'line';
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J'}; 
%     cfg.figsize = [450 350]; cfg.ylim = [-1650 -1440];
end;
% if strcmp(mode, 'onlineperf'), 
%     cfg.figsize = [450 400]; cfg.ylim = [-2700 -1400];
% end;
if any(strcmp(mode, {'stats', 'plot', 'onlineperf'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.alg = 'lspionline';
ocfg.trialtime = 1.5;           % sufficient for stabilization from most (all?) initial states
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.explordecay = 0.1^(1/ocfg.lspi_params.maxtime);  
ocfg.lspi_params.explor = 1;                % start with this exploration rate
ocfg.lspi_params.dstoretheta = 1;                          % store theta once every second
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'dupd', 'rep'};
ocfg.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
% perf eval
ocfg.forceeval = 1;                         % since no experiment actually converges

% --------------
% LSPI config
bcfg = cfg;
bcfg.alg = 'lspi';
bcfg.show = {'rep'};
bcfg.maxiter = 200;


expout = [];   % dummy

% ONLINE, RBF discrete experiment with finer storage of theta and also added per-sample updating
% 20 experiments available
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl';
rexp.N        = cfg.N; rexp.M = 3;
if isempty(cfg.dupd),   % a global setting overrides
    rexp.dupd     = [1 10 100 1000 5000] * 0.005;
    % rexp.dupd     = [10] * 0.005; rexp.plotminmax = 1;
end;
rexp.infix    = 'dth1';
rexp.addtocrt = [];
if cfg.useprefixes, rexp.expprefix = 'Online,';  end; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% ONLINE, RBF discrete experiment; the only difference from the above is that this is run on
% the new DCSC desktop
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl2';
rexp.N        = cfg.N; rexp.M = 3;
if isempty(cfg.dupd),   % a global setting overrides
    rexp.dupd     = [1 10 100 1000 5000] * 0.005;
    % rexp.dupd     = [10] * 0.005; rexp.plotminmax = 1;
end;
rexp.infix    = 'dupd2';
rexp.addtocrt = []; 
if cfg.useprefixes, rexp.expprefix = 'Online,';  end; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% ONLINE, RBF discrete experiment #3;
% run on the new DCSC desktop, and with an explor schedule that reaches .1 at T=200
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronlB';
rexp.N        = cfg.N; rexp.M = 3;
rexp.explordecay = 0.1^(1/200);  
if isempty(cfg.dupd),   % a global setting overrides
    rexp.dupd     = [1 10 100 1000 5000] * 0.005;
    % rexp.dupd     = [10] * 0.005; rexp.plotminmax = 1;
end;
rexp.infix    = 'ronlB';
rexp.addtocrt = []; 
if cfg.useprefixes, rexp.expprefix = 'Online,';  end; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% ONLINE, RBF discrete experiment #3; with restarts to fixed, control state
% run on the new DCSC desktop, and with an explor schedule that reaches .1 at T=200
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronlBfixed';
rexp.N        = cfg.N; rexp.M = 3;
rexp.explordecay = 0.1^(1/200);
rexp.lspi_params.reset = 'randfixed';
rexp.lspi_params.x0 = [-pi; 0];
rexp.lspi_params.resetper = 10;
if isempty(cfg.dupd),   % a global setting overrides
    rexp.dupd     = [1 10 100 1000 5000] * 0.005;
    % rexp.dupd     = [10] * 0.005; rexp.plotminmax = 1;
end;
rexp.infix    = 'ronlBfixed';
rexp.addtocrt = []; 
if cfg.useprefixes, rexp.expprefix = 'Online,';  end; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% BASELINE, offline RBF experiment
% 20 experiments available
rbexp = bcfg;
rbexp.type     = 'rbfdisc'; rbexp.miscinfo = 'rbaseline';
rbexp.N        = cfg.N; rbexp.M = 3;
rbexp.method   = 'randdisc';
rbexp.Ns       = cfg.Ns;  % many samples to get close to the best achievable solution
rbexp.addtocrt = expout; 
if cfg.useprefixes, rbexp.expprefix = 'Offline,'; end;
rbexp.infix    = 'dupd';
if checkrunexp(rbexp.miscinfo, exp), expout = lspiexp(rbexp); end;

% BASELINE, offline RBF experiment
% the only difference from the above is that this is run on
% the new DCSC desktop
rbexp = bcfg;
rbexp.type     = 'rbfdisc'; rbexp.miscinfo = 'rbaseline2';
rbexp.N        = cfg.N; rbexp.M = 3;
rbexp.method   = 'randdisc';
rbexp.Ns       = cfg.Ns;  % many samples to get close to the best achievable solution
rbexp.addtocrt = expout; 
if cfg.useprefixes, rbexp.expprefix = 'Offline,'; end;
rbexp.infix    = 'dupd2';
if checkrunexp(rbexp.miscinfo, exp), expout = lspiexp(rbexp); end;


% =============================
% Orthogonal poly experiments -- not used in a long time

% ONLINE RBF ORTHOPOLY
% 5 experiments available (not used in chapter)
roexp = ocfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'roonl';
roexp.N        = cfg.N; roexp.M = 2;
if isempty(cfg.dupd),   % a global setting overrides
    roexp.dupd     = [1 10 100 1000 5000] * 0.005;
    % roexp.dupd     = [10] * 0.005; roexp.plotminmax = 1;
end;
roexp.infix    = 'dth1';
roexp.addtocrt = []; 
if cfg.useprefixes, roexp.expprefix = 'Online RBF-ortho,';  end; 
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;

% BASELINE, offline RBF ORTHOPOLY experiment
% 5 experiments available (not used in chapter)
robexp = bcfg;
robexp.type     = 'rbfortho'; robexp.miscinfo = 'robaseline';
robexp.eps      = 1e-5;     % this one will never actually converge to 0
robexp.N        = cfg.N; robexp.M = 2;
robexp.method   = 'rand'; 
robexp.Ns       = cfg.Ns;  % many samples to get close to the best achievable solution
robexp.addtocrt = expout; 
if cfg.useprefixes, robexp.expprefix = 'Offline RBF-ortho,'; end;
robexp.infix    = 'dupd';
if checkrunexp(robexp.miscinfo, exp), expout = lspiexp(robexp); end;

% ----------------------------------------------------------------

% % Initial ONLINE, RBF discrete experiment -- no longer used (stored theta too rarely)
% rexp0 = ocfg;
% rexp0.type     = 'rbfdisc'; rexp0.miscinfo = 'ronl0';
% rexp0.N        = cfg.N; rexp0.M = 3;
% rexp0.lspi_params.dstoretheta = 15;       % store theta once every 15 seconds
% rexp0.donlineperf = [15:15:90 120:60:600]; 
% if isempty(cfg.dupd),   % a global setting overrides
%     % rexp0.dupd     = [10 50 100 200 1000] * 0.005;
%     rexp0.dupd     = [10 100 1000] * 0.005;    % online perf eval-ed for these only 
% end;
% rexp0.addtocrt = []; rexp0.expprefix = 'Online'; % only effective in stats+plot mode
% if checkrunexp(rexp0.miscinfo, exp), expout = lspiexp(rexp0); end;


end



