function lspios_explordecay(exp, mode, c)
% Real inverted pendulum. Using trajectories for samples

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'ronl'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
% --------------
% main LSPIEXP config
cfg.(mode) = 2;
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.98;
cfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
cfg.explordecay = [];
% misc config (output etc)
cfg.datadir = pwd;
cfg.eps = 0;                            % zero convergence threshold (force until real convergence)
cfg.useprefixes = 0;            % field currently unused (no prefixes)
% approximator main config
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.datadir = pwd;
% perf eval
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.fixedfigname = 0;
cfg.plottarget = 'screen'; cfg.savefig = 'ed';
cfg.plotminmax = 0; cfg.plotconf = 0;
cfg.minmaxtype = 'area';
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J'}; 
%     cfg.figsize = [450 350];
end;
% if strcmp(mode, 'onlineperf'), 
%     cfg.ylim = [-2400 -1400];
% end;
if any(strcmp(mode, {'stats', 'plot', 'onlineperf'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.alg = 'lspionline';
ocfg.trialtime = 1.5; 
ocfg.dupd = 10 * 0.005;                     % this appears to be among the best in the 
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.lspi_params.explor = 1;                % start with this exploration rate
    % decay so that we have 0.1 at the end of the experiment
ocfg.lspi_params.dstoretheta = 1;           % store theta once every second
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'explordecay', 'rep'};
% perf eval
ocfg.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
ocfg.forceeval = 1;                         % since no experiment actually converges

expout = [];   % dummy

% ONLINE, RBF discrete experiment
% 20 exps available
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl';
rexp.N        = 11; rexp.M = 3;
% these are all the experiments for which data is available
if isempty(cfg.explordecay),
    rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.5^(1/600) 0.8^(1/600)];
    % this is used for performance eval (plotting); the above is too dense and unreadable
    % rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.8^(1/600)];
end;
% rexp.plotminmax = 1;
rexp.addtocrt = []; rexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% ONLINE, RBF discrete experiment; the differences from the above are that this is run on
% the new DCSC desktop; and that there is also a 200-sec e=0.1 experiment
% available
% 20 experiments available
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl2';
rexp.N        = 11; rexp.M = 3;
if isempty(cfg.explordecay),
    % these are all the experiments for which data is available
    rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 200 300 600])  0.5^(1/600) 0.8^(1/600)];
    % this is used for performance eval (plotting); the above is too dense and unreadable
    rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 200 300 600])  0.5^(1/600)];
end;
% rexp.plotminmax = 1;
rexp.infix    = 'ed2';
rexp.addtocrt = []; rexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% ONLINE, RBF discrete experiment; with "control" resets to fixed initial
% states, once every 10 trials
% run on the new DCSC desktop
% 20 experiments available
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'fixed';
rexp.N        = 11; rexp.M = 3;
rexp.lspi_params.reset = 'randfixed';
rexp.lspi_params.x0 = [-pi; 0];
rexp.lspi_params.resetper = 10;
if isempty(cfg.explordecay),
    % these are all the experiments for which data is available
    rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 200 300 600])  0.5^(1/600) 0.8^(1/600)];
    % less dense for plotting
    rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 200 300 600])  0.5^(1/600)];
end;
rexp.infix    = 'ed2fixed';
rexp.addtocrt = []; rexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;



% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Below follow older experiments

% ONLINE, RBF ortho experiment
% data available for 5 exps (not used in chapter / paper / etc)
rexp = ocfg;
rexp.type     = 'rbfortho'; rexp.miscinfo = 'roonl';
rexp.N        = 11; rexp.M = 2;
if isempty(cfg.explordecay)
    % these are all the experiments for which data is available
    rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.5^(1/600) 0.8^(1/600)];
    % % this is used for performance eval (plotting); the above is too dense and unreadable
    % rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.8^(1/600)];
end;
% rexp.plotminmax = 1;
rexp.addtocrt = []; rexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


end

