function out = linsys_problem(what, varargin)
% Discretized linear system.
% For now, a 2nd order system of the form 1/s(Ts+1) is hardcoded
%   OUT = LINSYS_PROBLEM(WHAT)
% This function conforms to the specifications established by SAMPLE_PROBLEM.

maxx = [10 5]';
maxu = 2;
maxk = [0.5 0.5]';
% just defaults, not guaranteed to give a feasible solution
xgrids = {-maxx(1):maxx(1), -maxx(2):maxx(2)};
ugrids = {[-maxu 0 maxu]};
x0 = (2*rand) * maxx - maxx;
gamma = 0.98;

switch what
    case 'model'
        
        % system type
        if nargin < 2,  type = 'ls';        % only 1 system is implemented, 1/s(Ts+1)
        else            type = varargin{1};
        end;
        % control type
        if nargin < 3,  control = 'direct';
        else            control = varargin{2};
        end;
        
        m.maxx = maxx;
        m.p = 2;
        % parameters
        T = 0.5;
        Ts = 0.1;
        % discretized SS model
        G = ss(tf(1, conv([1 0], [T 1])));
        G = c2d(G, Ts, 'zoh');  % discrete-time model
        
        m.Ts = Ts;
        m.G = G;
        m.A = G.a; m.B = G.b;
        m.C = G.c; m.D = G.d;

        switch control,
            case 'direct',          % direct control
                m.maxu = maxu;
                m.q = 1;
                m.fun = @linsys_mdp;
            case 'lsf',             % linear state feedback
                m.maxcmd = maxu;    % maximum direct control
                m.maxu = maxk;
                m.q = 2;
                m.fun = @linsys_lsf_mdp;
        end;

        % reward specification
        % config 1: band reward
        m.rewtype = 'band';
        m.zeroband = [0.2 0.2]';
        m.zeroreward = 1;
        m.maxr = 1;

        out = m;

    case 'fuzzy'
        cfg.xgrids = xgrids;
        cfg.ugrids = ugrids;
        cfg.gamma = gamma;
        cfg.x0 = x0;
    
        out = cfg;
        
end;


% END doubleint_problem(), RETURNING out ====================================