% Script to plot 1-D triangular partition example 
% and 2-D composition (pyramids) of 1-D triangular part
% Harcoded for a 3-level partition of 2-D velocity for the navigation problem
% Some parts are hardcoded for 3x3 MFs (e.g. intensity matrices)
% but most parts (like the surface/plot computations) are not
% Note that more than 3x3 MFs are very difficult to read
%
% Config adapted for ALAMAS-07 paper

cfg.samefig = 0;
cfg.plottarget = 'latex';
cfg.savedir = 'D:\Work\tex\papers\alamas07\img\';
cfg.savefig = 'nav_velocitymfs';
cfg.onedimfig = 'nav_1d_velocitymfs';
cfg.twodimfig = 'nav_2d_velocitymfs';
cfg.figsize = [300 300];

% init
C = [-2 0 2];
mfs = gen_mfs(C);

% plot
figh = [];

% Prepare figure 1
if cfg.samefig, 
    figh(end+1) = figurex(cfg.figsize);
    subplot(121); 
else
    figh(end+1) = figurex(cfg.figsize - [50 120]);
end;

% -------------------------
% Figure 1: 1-D MFs
commonprop = {'Interpreter','LaTeX','FontSize',12};

lsty = {{'k-'}, {'k:'}, {'k-.'}};
Cx = [C(1) C C(end)];
for i = 1:length(C),
    % compute the vector of values of MF#i in all the points in domain #i
    d = Cx(i:i+2); mu = 0*d;
    for j = 2:length(d-1),
        m = mdegs(d(j), mfs);
        mu(j) = m(i);
    end;
    mu(1) = 0; mu(end) = 0;
    plot(d, mu, lsty{i}{:}, 'LineWidth', 2); hold on;
end;
xlabel('DOTCX'); 
ylabel('MUDOTCX');

% Close figure 1, prepare figure 2
if cfg.samefig, subplot(122); 
else
%     saveplot(figh(end), [cfg.savedir cfg.onedimfig], cfg.plottarget);
    figh(end+1) = figurex(cfg.figsize);
end;

% -------------------------
% Figure 2: 2-D composition
colormap(gray);
Np = 14;
eint = [0, .8, .3;
       .6, 0, .6; 
       .3, .8, 0];
fint = [.8, .5, .8; 
        .8, .3, .8;
        .8, .5, .8];
for i = 1:length(C),
    for j =1:length(C),
        % 2-D domain of composite MF i,j
        d = [Cx(i) : (Cx(i+2)-Cx(i))/Np : Cx(i+2)
             Cx(j) : (Cx(j+2)-Cx(j))/Np : Cx(j+2)];
        mu = d*0;
        for i1 = 1:size(d, 2),
            m1 = mdegs(d(1, i1), mfs); m1 = m1(i);
            for j1 = 1:size(d, 2),
                m2 = mdegs(d(2, j1), mfs); m2 = m2(j);
                mu(i1, j1) = m1 * m2;
            end;
        end;
        mesh(d(1, :), d(2, :), mu', 'EdgeColor', eint(i,j)*[1 1 1], ...
            'FaceColor', fint(i,j)*[1 1 1], 'FaceAlpha', 1, 'EdgeAlpha', 1); hold on;
    end;
end;

% draw edges
mux1 = 0*Cx; mux2 = 0*C; 
mux1(2:2:end-1) = 1;
mux2(2:2:end-1) = 1;
esty = {'LineWidth', 2, 'Color', 'k', 'LineStyle', '-'};
line(Cx, -2+0*Cx, mux1, esty{:}); line(C, -2+0*C, mux2, esty{:});
line(-2+0*Cx, Cx, mux1, esty{:}); line(-2+0*C, C, mux2, esty{:});

xlabel('$\dot{c}_x$', commonprop{:});
ylabel('$\dot{c}_y$', commonprop{:}); 
zlabel('$\mu(\dot{c}_x\ , \dot{c}_y)$', commonprop{:}); 

if cfg.samefig, 
    saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
else
    saveplot(figh(end), [cfg.savedir cfg.twodimfig], cfg.plottarget);
end;

% pause and close
pause; close(figh);