%% Optimistic planning experiments for the inverted pendulum
function resarm_soop(mode, expids, custombcfg, customacfg, customplanparams)

%% General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
% BCFG.idfields = {'alg', 'problem', 'planalg', 'planparamsn'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithm
ACFG = struct;
ACFG.problem = 'resonatingarm_problem';
ACFG.model_params = {'rewtype=exp'};
ACFG.online = 1;
ACFG.gamma = 0.95;
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 100;
ACFG.reset = [-.85, 0]';
% visualization
ACFG.verb = 3;
ACFG.datadir = pwd; %'c:\mfiles\approxrl\systems\invpend_setup\soop_ip';
ACFG.online = 1;

cd(ACFG.datadir); 

% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;
if nargin < 4, customplanparams = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            acfg.planparams = copyfields(customplanparams, acfg.planparams);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            else 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

%% SOOP // CRAN Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'soop';
bcfg.legendprefix = 'SOOP';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsdimselect', 'planparamsalpha'};

acfg = ACFG;
acfg.planalg = 'sooplp';
acfg.planparams.method = 'soop';
acfg.planparams.n = [500 1000 5000]; %[500 1000 5000 10000 15000];
acfg.planparams.dimselect = 'disc';
acfg.planparams.alpha = 0.1:0.1:0.9;

execbatchexp(bcfg.idinfix);

%% LP // CRAN Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'lp';
bcfg.legendprefix = 'LP';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamslipschitz'};

acfg = ACFG;
acfg.planalg = 'sooplp';
acfg.planparams.method = 'lp';
acfg.planparams.dimselect = 'lp';
acfg.planparams.n = [500 1000 5000]; %[500 1000 5000 10000 15000];
acfg.planparams.lipschitz = 0.1:0.1:1.5;

execbatchexp(bcfg.idinfix);


%% HOLOP // CRAN Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'holop';
bcfg.legendprefix = 'HOLOP';
bcfg.in = 'planparamsn';
bcfg.startat = 1;
bcfg.repeat = 10;
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsK', 'rep'};

acfg = ACFG;
acfg.planalg = 'holop';
acfg.planparams.n = [500 1000 5000]; %[500 1000 5000 10000 15000];
acfg.planparams.K = [5 10 15 20 25 30 40 50 75 100];
acfg.planparams.detact = 0;

execbatchexp(bcfg.idinfix);

%% OPD, discrete-action baseline // CRAN Desktop
bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'opd';
bcfg.legendprefix = 'OPD';
bcfg.in = 'planparamsntrans';
bcfg.idfields = {'alg', 'problem', 'planparamsntrans', 'planparamsU'};

acfg = ACFG;
acfg.planalg = 'opd';
acfg.planparams.ntrans = [500 1000 5000]; %[500 1000 5000 10000 15000];
% form a set of equidistant action discretizations
M = 3:2:15;
U = cell(length(M), 1); for i = 1:length(M), U{i} = symequidgrid(M(i), 2); end; % 2 = max input
acfg.planparams.U = U;

execbatchexp(bcfg.idinfix);

end     % main function