function wdata = tiling_reset(tdim, cfg)
% TILING_RESET Resets parameter matrix of tiling
%   WDATA = TILING_RESET(TDIM, CFG)
% Parameters:
%   tdim        - dimensionality information for the tiling
%   cfg         - additional config, structure with fields (defaults in square brackets)
%       init        - initialization of tile weights: {0}, another fixed numeric value, or 'random'
%       sigma       - random standard deviation for random tile init
%
% Returns:
%   xdata       - grid data, a cell array containing matrices of tile boundaries, one per dimension
%   wdata       - tiling weight data
%   tdim        - dimensionality information for the tiling


% ===== INPUT PARAMETERS ====

% default config
CFG.init = 0;
CFG.sigma = 1;

if isscalar(cfg.init),
    wdata = cfg.init + zeros(prod(tdim.tsize), prod(tdim.dsize));
elseif strcmp(cfg.init, 'random'),
    wdata = cfg.sigma * rand(prod(tdim.tsize), prod(tdim.dsize));
else
    error('Unsupported initialization type');
end;

% === END tiling_reset() RETURNING wdata ========================================
